@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # This file is part of beets.
 # Copyright 2015, Adrian Sampson.
 #
@@ -511,7 +512,10 @@ def album_for_mbid(release_id):
     if the ID is not found.
     """
     try:
-        return mb.album_for_id(release_id)
+        album = mb.album_for_id(release_id)
+        if album:
+            plugins.send('albuminfo_received', info=album)
+        return album
     except mb.MusicBrainzAPIError as exc:
         exc.log(log)
 
@@ -521,7 +525,10 @@ def track_for_mbid(recording_id):
     if the ID is not found.
     """
     try:
-        return mb.track_for_id(recording_id)
+        track = mb.track_for_id(recording_id)
+        if track:
+            plugins.send('trackinfo_received', info=track)
+        return track
     except mb.MusicBrainzAPIError as exc:
         exc.log(log)
 
@@ -529,14 +536,20 @@ def track_for_mbid(recording_id):
 def albums_for_id(album_id):
     """Get a list of albums for an ID."""
     candidates = [album_for_mbid(album_id)]
-    candidates.extend(plugins.album_for_id(album_id))
+    plugin_albums = plugins.album_for_id(album_id)
+    for a in plugin_albums:
+        plugins.send('albuminfo_received', info=a)
+    candidates.extend(plugin_albums)
     return filter(None, candidates)
 
 
 def tracks_for_id(track_id):
     """Get a list of tracks for an ID."""
     candidates = [track_for_mbid(track_id)]
-    candidates.extend(plugins.track_for_id(track_id))
+    plugin_tracks = plugins.track_for_id(track_id)
+    for t in plugin_tracks:
+        plugins.send('trackinfo_received', info=t)
+    candidates.extend(plugin_tracks)
     return filter(None, candidates)
 
 
@@ -566,6 +579,10 @@ def album_candidates(items, artist, album, va_likely):
     # Candidates from plugins.
     out.extend(plugins.candidates(items, artist, album, va_likely))
 
+    # Notify subscribed plugins about fetched album info
+    for a in out:
+        plugins.send('albuminfo_received', info=a)
+
     return out
 
 
@@ -586,4 +603,8 @@ def item_candidates(item, artist, title):
     # Plugin candidates.
     out.extend(plugins.item_candidates(item, artist, title))
 
+    # Notify subscribed plugins about fetched track info
+    for i in out:
+        plugins.send('trackinfo_received', info=i)
+
     return out
