@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # This file is part of beets.
 # Copyright 2015, Adrian Sampson.
 #
@@ -21,10 +22,12 @@ from __future__ import (division, absolute_import, print_function,
 
 import os
 import re
+from collections import namedtuple, Counter
+from itertools import chain
 
 import beets
 from beets import ui
-from beets.ui import print_, input_, decargs
+from beets.ui import print_, input_, decargs, show_path_changes
 from beets import autotag
 from beets.autotag import Recommendation
 from beets.autotag import hooks
@@ -38,6 +41,7 @@ from beets import logging
 from beets.util.confit import _package_path
 
 VARIOUS_ARTISTS = u'Various Artists'
+PromptChoice = namedtuple('ExtraChoice', ['short', 'long', 'callback'])
 
 # Global logger.
 log = logging.getLogger('beets')
@@ -82,29 +86,11 @@ def fields_func(lib, opts, args):
         names.sort()
         print_("  " + "\n  ".join(names))
 
-    def _show_plugin_fields(album):
-        plugin_fields = []
-        for plugin in plugins.find_plugins():
-            if album:
-                fdict = plugin.album_template_fields
-            else:
-                fdict = plugin.template_fields
-            plugin_fields += fdict.keys()
-        if plugin_fields:
-            print_("Template fields from plugins:")
-            _print_rows(plugin_fields)
-
     print_("Item fields:")
-    _print_rows(library.Item._fields.keys() +
-                library.Item._getters().keys() +
-                library.Item._types.keys())
-    _show_plugin_fields(False)
+    _print_rows(library.Item.all_keys())
 
-    print_("\nAlbum fields:")
-    _print_rows(library.Album._fields.keys() +
-                library.Album._getters().keys() +
-                library.Album._types.keys())
-    _show_plugin_fields(True)
+    print_("Album fields:")
+    _print_rows(library.Album.all_keys())
 
 
 fields_cmd = ui.Subcommand(
@@ -488,7 +474,8 @@ def _summary_judment(rec):
 
 
 def choose_candidate(candidates, singleton, rec, cur_artist=None,
-                     cur_album=None, item=None, itemcount=None):
+                     cur_album=None, item=None, itemcount=None,
+                     extra_choices=[]):
     """Given a sorted list of candidates, ask the user for a selection
     of which candidate to use. Applies to both full albums and
     singletons  (tracks). Candidates are either AlbumMatch or TrackMatch
@@ -496,8 +483,16 @@ def choose_candidate(candidates, singleton, rec, cur_artist=None,
     `cur_album`, and `itemcount` must be provided. For singletons,
     `item` must be provided.
 
-    Returns the result of the choice, which may SKIP, ASIS, TRACKS, or
-    MANUAL or a candidate (an AlbumMatch/TrackMatch object).
+    `extra_choices` is a list of `PromptChoice`s, containg the choices
+    appended by the plugins after receiving the `before_choose_candidate`
+    event. If not empty, the choices are appended to the prompt presented
+    to the user.
+
+    Returns one of the following:
+    * the result of the choice, which may be SKIP, ASIS, TRACKS, or MANUAL
+    * a candidate (an AlbumMatch/TrackMatch object)
+    * the short letter of a `PromptChoice` (if the user selected one of
+    the `extra_choices`).
     """
     # Sanity check.
     if singleton:
@@ -506,6 +501,10 @@ def choose_candidate(candidates, singleton, rec, cur_artist=None,
         assert cur_artist is not None
         assert cur_album is not None
 
+    # Build helper variables for extra choices.
+    extra_opts = tuple(c.long for c in extra_choices)
+    extra_actions = tuple(c.short for c in extra_choices)
+
     # Zero candidates.
     if not candidates:
         if singleton:
@@ -519,7 +518,7 @@ def choose_candidate(candidates, singleton, rec, cur_artist=None,
                    'http://beets.readthedocs.org/en/latest/faq.html#nomatch')
             opts = ('Use as-is', 'as Tracks', 'Group albums', 'Skip',
                     'Enter search', 'enter Id', 'aBort')
-        sel = ui.input_options(opts)
+        sel = ui.input_options(opts + extra_opts)
         if sel == 'u':
             return importer.action.ASIS
         elif sel == 't':
@@ -535,6 +534,8 @@ def choose_candidate(candidates, singleton, rec, cur_artist=None,
             return importer.action.MANUAL_ID
         elif sel == 'g':
             return importer.action.ALBUMS
+        elif sel in extra_actions:
+            return sel
         else:
             assert False
 
@@ -588,7 +589,8 @@ def choose_candidate(candidates, singleton, rec, cur_artist=None,
             else:
                 opts = ('Skip', 'Use as-is', 'as Tracks', 'Group albums',
                         'Enter search', 'enter Id', 'aBort')
-            sel = ui.input_options(opts, numrange=(1, len(candidates)))
+            sel = ui.input_options(opts + extra_opts,
+                                   numrange=(1, len(candidates)))
             if sel == 's':
                 return importer.action.SKIP
             elif sel == 'u':
@@ -606,6 +608,8 @@ def choose_candidate(candidates, singleton, rec, cur_artist=None,
                 return importer.action.MANUAL_ID
             elif sel == 'g':
                 return importer.action.ALBUMS
+            elif sel in extra_actions:
+                return sel
             else:  # Numerical selection.
                 match = candidates[sel - 1]
                 if sel != 1:
@@ -640,7 +644,8 @@ def choose_candidate(candidates, singleton, rec, cur_artist=None,
         })
         if default is None:
             require = True
-        sel = ui.input_options(opts, require=require, default=default)
+        sel = ui.input_options(opts + extra_opts, require=require,
+                               default=default)
         if sel == 'a':
             return match
         elif sel == 'g':
@@ -658,6 +663,8 @@ def choose_candidate(candidates, singleton, rec, cur_artist=None,
             raise importer.ImportAbort()
         elif sel == 'i':
             return importer.action.MANUAL_ID
+        elif sel in extra_actions:
+            return sel
 
 
 def manual_search(singleton):
@@ -701,10 +708,14 @@ class TerminalImportSession(importer.ImportSession):
         # Loop until we have a choice.
         candidates, rec = task.candidates, task.rec
         while True:
+            # Gather extra choices from plugins.
+            extra_choices = self._get_plugin_choices(task)
+            extra_ops = {c.short: c.callback for c in extra_choices}
+
             # Ask for a choice from the user.
             choice = choose_candidate(
                 candidates, False, rec, task.cur_artist, task.cur_album,
-                itemcount=len(task.items)
+                itemcount=len(task.items), extra_choices=extra_choices
             )
 
             # Choose which tags to use.
@@ -725,6 +736,12 @@ class TerminalImportSession(importer.ImportSession):
                     _, _, candidates, rec = autotag.tag_album(
                         task.items, search_id=search_id
                     )
+            elif choice in extra_ops.keys():
+                # Allow extra ops to automatically set the post-choice.
+                post_choice = extra_ops[choice](self, task)
+                if isinstance(post_choice, importer.action):
+                    # MANUAL and MANUAL_ID have no effect, even if returned.
+                    return post_choice
             else:
                 # We have a candidate! Finish tagging. Here, choice is an
                 # AlbumMatch object.
@@ -749,8 +766,12 @@ class TerminalImportSession(importer.ImportSession):
             return action
 
         while True:
+            extra_choices = self._get_plugin_choices(task)
+            extra_ops = {c.short: c.callback for c in extra_choices}
+
             # Ask for a choice.
-            choice = choose_candidate(candidates, True, rec, item=task.item)
+            choice = choose_candidate(candidates, True, rec, item=task.item,
+                                      extra_choices=extra_choices)
 
             if choice in (importer.action.SKIP, importer.action.ASIS):
                 return choice
@@ -767,6 +788,12 @@ class TerminalImportSession(importer.ImportSession):
                 if search_id:
                     candidates, rec = autotag.tag_item(task.item,
                                                        search_id=search_id)
+            elif choice in extra_ops.keys():
+                # Allow extra ops to automatically set the post-choice.
+                post_choice = extra_ops[choice](self, task)
+                if isinstance(post_choice, importer.action):
+                    # MANUAL and MANUAL_ID have no effect, even if returned.
+                    return post_choice
             else:
                 # Chose a candidate.
                 assert isinstance(choice, autotag.TrackMatch)
@@ -779,16 +806,6 @@ class TerminalImportSession(importer.ImportSession):
         log.warn(u"This {0} is already in the library!",
                  ("album" if task.is_album else "item"))
 
-        # skip empty albums (coming from a previous failed import session)
-        if task.is_album:
-            real_duplicates = [dup for dup in found_duplicates if dup.items()]
-            if not real_duplicates:
-                log.info("All duplicates are empty, we ignore them")
-                task.should_remove_duplicates = True
-                return
-        else:
-            real_duplicates = found_duplicates
-
         if config['import']['quiet']:
             # In quiet mode, don't prompt -- just skip.
             log.info(u'Skipping.')
@@ -796,17 +813,12 @@ class TerminalImportSession(importer.ImportSession):
         else:
             # Print some detail about the existing and new items so the
             # user can make an informed decision.
-            for duplicate in real_duplicates:
+            for duplicate in found_duplicates:
                 print_("Old: " + summarize_items(
                     list(duplicate.items()) if task.is_album else [duplicate],
                     not task.is_album,
                 ))
 
-            if real_duplicates != found_duplicates:  # there's empty albums
-                count = len(found_duplicates) - len(real_duplicates)
-                print_("Old: {0} empty album{1}".format(
-                       count, "s" if count > 1 else ""))
-
             print_("New: " + summarize_items(
                 task.imported_items(),
                 not task.is_album,
@@ -833,6 +845,48 @@ class TerminalImportSession(importer.ImportSession):
                            "was interrupted. Resume (Y/n)?"
                            .format(displayable_path(path)))
 
+    def _get_plugin_choices(self, task):
+        """Get the extra choices appended to the plugins to the ui prompt.
+
+        The `before_choose_candidate` event is sent to the plugins, with
+        session and task as its parameters. Plugins are responsible for
+        checking the right conditions and returning a list of `PromptChoice`s,
+        which is flattened and checked for conflicts.
+
+        Raises `ValueError` if two of the choices have the same short letter.
+
+        Returns a list of `PromptChoice`s.
+        """
+        # Send the before_choose_candidate event and flatten list.
+        extra_choices = list(chain(*plugins.send('before_choose_candidate',
+                                                 session=self, task=task)))
+        # Take into account default options, for duplicate checking.
+        all_choices = [PromptChoice('a', 'Apply', None),
+                       PromptChoice('s', 'Skip', None),
+                       PromptChoice('u', 'Use as-is', None),
+                       PromptChoice('t', 'as Tracks', None),
+                       PromptChoice('g', 'Group albums', None),
+                       PromptChoice('e', 'Enter search', None),
+                       PromptChoice('i', 'enter Id', None),
+                       PromptChoice('b', 'aBort', None)] +\
+            extra_choices
+
+        short_letters = [c.short for c in all_choices]
+        if len(short_letters) != len(set(short_letters)):
+            # Duplicate short letter has been found.
+            duplicates = [i for i, count in Counter(short_letters).items()
+                          if count > 1]
+            for short in duplicates:
+                # Keep the first of the choices, removing the rest.
+                dup_choices = [c for c in all_choices if c.short == short]
+                for c in dup_choices[1:]:
+                    log.warn(u"Prompt choice '{0}' removed due to conflict "
+                             u"with '{1}' (short letter: '{2}')",
+                             c.long, dup_choices[0].long, c.short)
+                    extra_choices.remove(c)
+        return extra_choices
+
+
 # The import command.
 
 
@@ -991,6 +1045,8 @@ def list_func(lib, opts, args):
 
 
 list_cmd = ui.Subcommand('list', help='query the library', aliases=('ls',))
+list_cmd.parser.usage += "\n" \
+    'Example: %prog -f \'$album: $title\' artist:beatles'
 list_cmd.parser.add_all_common_options()
 list_cmd.func = list_func
 default_commands.append(list_cmd)
@@ -1086,7 +1142,8 @@ def update_items(lib, query, album, move, pretend):
 
 
 def update_func(lib, opts, args):
-    update_items(lib, decargs(args), opts.album, opts.move, opts.pretend)
+    update_items(lib, decargs(args), opts.album, ui.should_move(opts.move),
+                 opts.pretend)
 
 
 update_cmd = ui.Subcommand(
@@ -1095,7 +1152,11 @@ update_cmd = ui.Subcommand(
 update_cmd.parser.add_album_option()
 update_cmd.parser.add_format_option()
 update_cmd.parser.add_option(
-    '-M', '--nomove', action='store_false', default=True, dest='move',
+    '-m', '--move', action='store_true', dest='move',
+    help="move files in the library directory"
+)
+update_cmd.parser.add_option(
+    '-M', '--nomove', action='store_false', dest='move',
     help="don't move files in library"
 )
 update_cmd.parser.add_option(
@@ -1119,11 +1180,12 @@ def remove_items(lib, query, album, delete):
     print_()
     if delete:
         fmt = u'$path - $title'
-        prompt = 'Really DELETE %i files (y/n)?' % len(items)
+        prompt = 'Really DELETE %i file%s (y/n)?' % \
+                 (len(items), 's' if len(items) > 1 else '')
     else:
         fmt = ''
-        prompt = 'Really remove %i items from the library (y/n)?' % \
-                 len(items)
+        prompt = 'Really remove %i item%s from the library (y/n)?' % \
+                 (len(items), 's' if len(items) > 1 else '')
 
     # Show all the items.
     for item in items:
@@ -1323,17 +1385,19 @@ def modify_func(lib, opts, args):
     query, mods, dels = modify_parse_args(decargs(args))
     if not mods and not dels:
         raise ui.UserError('no modifications specified')
-    write = opts.write if opts.write is not None else \
-        config['import']['write'].get(bool)
-    modify_items(lib, mods, dels, query, write, opts.move, opts.album,
-                 not opts.yes)
+    modify_items(lib, mods, dels, query, ui.should_write(opts.write),
+                 ui.should_move(opts.move), opts.album, not opts.yes)
 
 
 modify_cmd = ui.Subcommand(
     'modify', help='change metadata fields', aliases=('mod',)
 )
 modify_cmd.parser.add_option(
-    '-M', '--nomove', action='store_false', default=True, dest='move',
+    '-m', '--move', action='store_true', dest='move',
+    help="move files in the library directory"
+)
+modify_cmd.parser.add_option(
+    '-M', '--nomove', action='store_false', dest='move',
     help="don't move files in library"
 )
 modify_cmd.parser.add_option(
@@ -1356,7 +1420,7 @@ default_commands.append(modify_cmd)
 
 # move: Move/copy files to the library or a new base directory.
 
-def move_items(lib, dest, query, copy, album):
+def move_items(lib, dest, query, copy, album, pretend):
     """Moves or copies items to a new base directory, given by dest. If
     dest is None, then the library's base directory is used, making the
     command "consolidate" files.
@@ -1366,12 +1430,21 @@ def move_items(lib, dest, query, copy, album):
 
     action = 'Copying' if copy else 'Moving'
     entity = 'album' if album else 'item'
-    log.info(u'{0} {1} {2}s.', action, len(objs), entity)
-    for obj in objs:
-        log.debug(u'moving: {0}', util.displayable_path(obj.path))
+    log.info(u'{0} {1} {2}{3}.', action, len(objs), entity,
+             's' if len(objs) > 1 else '')
+    if pretend:
+        if album:
+            show_path_changes([(item.path, item.destination(basedir=dest))
+                               for obj in objs for item in obj.items()])
+        else:
+            show_path_changes([(obj.path, obj.destination(basedir=dest))
+                               for obj in objs])
+    else:
+        for obj in objs:
+            log.debug(u'moving: {0}', util.displayable_path(obj.path))
 
-        obj.move(copy, basedir=dest)
-        obj.store()
+            obj.move(copy, basedir=dest)
+            obj.store()
 
 
 def move_func(lib, opts, args):
@@ -1381,7 +1454,7 @@ def move_func(lib, opts, args):
         if not os.path.isdir(dest):
             raise ui.UserError('no such directory: %s' % dest)
 
-    move_items(lib, dest, decargs(args), opts.copy, opts.album)
+    move_items(lib, dest, decargs(args), opts.copy, opts.album, opts.pretend)
 
 
 move_cmd = ui.Subcommand(
@@ -1395,6 +1468,9 @@ move_cmd.parser.add_option(
     '-c', '--copy', default=False, action='store_true',
     help='copy instead of moving'
 )
+move_cmd.parser.add_option(
+    '-p', '--pretend', default=False, action='store_true',
+    help='show how files would be moved, but don\'t touch anything')
 move_cmd.parser.add_album_option()
 move_cmd.func = move_func
 default_commands.append(move_cmd)
@@ -1481,12 +1557,14 @@ def config_func(lib, opts, args):
 
 def config_edit():
     """Open a program to edit the user configuration.
+    An empty config file is created if no existing config file exists.
     """
     path = config.user_config_path()
-
-    editor = os.environ.get('EDITOR')
+    editor = util.editor_command()
     try:
-        util.interactive_open(path, editor)
+        if not os.path.isfile(path):
+            open(path, 'w+').close()
+        util.interactive_open([path], editor)
     except OSError as exc:
         message = "Could not edit configuration: {0}".format(exc)
         if not editor:
