@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # This file is part of beets.
 # Copyright 2015, Jakob Schnitzer.
 #
@@ -20,7 +21,6 @@ from __future__ import (division, absolute_import, print_function,
 from beets.plugins import BeetsPlugin
 from beets import autotag, library, ui, util
 from beets.autotag import hooks
-from beets import config
 from collections import defaultdict
 
 
@@ -46,11 +46,14 @@ class MBSyncPlugin(BeetsPlugin):
                             help='update metadata from musicbrainz')
         cmd.parser.add_option('-p', '--pretend', action='store_true',
                               help='show all changes but do nothing')
+        cmd.parser.add_option('-m', '--move', action='store_true',
+                              dest='move',
+                              help="move files in the library directory")
         cmd.parser.add_option('-M', '--nomove', action='store_false',
-                              default=True, dest='move',
+                              dest='move',
                               help="don't move files in library")
         cmd.parser.add_option('-W', '--nowrite', action='store_false',
-                              default=config['import']['write'], dest='write',
+                              default=None, dest='write',
                               help="don't write updated metadata to files")
         cmd.parser.add_format_option()
         cmd.func = self.func
@@ -59,9 +62,9 @@ class MBSyncPlugin(BeetsPlugin):
     def func(self, lib, opts, args):
         """Command handler for the mbsync function.
         """
-        move = opts.move
+        move = ui.should_move(opts.move)
         pretend = opts.pretend
-        write = opts.write
+        write = ui.should_write(opts.write)
         query = ui.decargs(args)
 
         self.singletons(lib, query, move, pretend, write)
@@ -137,6 +140,7 @@ class MBSyncPlugin(BeetsPlugin):
                             break
 
             # Apply.
+            self._log.debug('applying changes to {}', album_formatted)
             with lib.transaction():
                 autotag.apply_metadata(album_info, mapping)
                 changed = False
