@@ -1,5 +1,6 @@
+# -*- coding: utf-8 -*-
 # This file is part of beets.
-# Copyright 2013, Pedro Silva.
+# Copyright 2015, Pedro Silva.
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -14,40 +15,20 @@
 
 """List missing tracks.
 """
-import logging
+from __future__ import (division, absolute_import, print_function,
+                        unicode_literals)
 
 from beets.autotag import hooks
 from beets.library import Item
 from beets.plugins import BeetsPlugin
-from beets.ui import decargs, print_obj, Subcommand
-
-PLUGIN = 'missing'
-log = logging.getLogger('beets')
+from beets.ui import decargs, print_, Subcommand
+from beets import config
 
 
 def _missing_count(album):
     """Return number of missing items in `album`.
     """
-    return (album.tracktotal or 0) - len(album.items())
-
-
-def _missing(album):
-    """Query MusicBrainz to determine items missing from `album`.
-    """
-    item_mbids = map(lambda x: x.mb_trackid, album.items())
-
-    if len([i for i in album.items()]) < album.tracktotal:
-        # fetch missing items
-        # TODO: Implement caching that without breaking other stuff
-        album_info = hooks.album_for_mbid(album.mb_albumid)
-        for track_info in getattr(album_info, 'tracks', []):
-            if track_info.track_id not in item_mbids:
-                item = _item(track_info, album_info, album.id)
-                log.debug(u'{0}: track {1} in album {2}'
-                          .format(PLUGIN,
-                                  track_info.track_id,
-                                  album_info.album_id))
-                yield item
+    return (album.albumtotal or 0) - len(album.items())
 
 
 def _item(track_info, album_info, album_id):
@@ -105,7 +86,6 @@ class MissingPlugin(BeetsPlugin):
         super(MissingPlugin, self).__init__()
 
         self.config.add({
-            'format': None,
             'count': False,
             'total': False,
         })
@@ -115,26 +95,20 @@ class MissingPlugin(BeetsPlugin):
         self._command = Subcommand('missing',
                                    help=__doc__,
                                    aliases=['miss'])
-
-        self._command.parser.add_option('-f', '--format', dest='format',
-                                        action='store', type='string',
-                                        help='print with custom FORMAT',
-                                        metavar='FORMAT')
-
         self._command.parser.add_option('-c', '--count', dest='count',
                                         action='store_true',
                                         help='count missing tracks per album')
-
         self._command.parser.add_option('-t', '--total', dest='total',
                                         action='store_true',
                                         help='count total of missing tracks')
+        self._command.parser.add_format_option()
 
     def commands(self):
         def _miss(lib, opts, args):
             self.config.set_args(opts)
-            fmt = self.config['format'].get()
             count = self.config['count'].get()
             total = self.config['total'].get()
+            fmt = config['format_album' if count else 'format_item'].get()
 
             albums = lib.albums(decargs(args))
             if total:
@@ -142,18 +116,32 @@ class MissingPlugin(BeetsPlugin):
                 return
 
             # Default format string for count mode.
-            if count and not fmt:
-                fmt = '$albumartist - $album: $missing'
+            if count:
+                fmt += ': $missing'
 
             for album in albums:
                 if count:
-                    missing = _missing_count(album)
-                    if missing:
-                        print_obj(album, lib, fmt=fmt)
+                    if _missing_count(album):
+                        print_(format(album, fmt))
 
                 else:
-                    for item in _missing(album):
-                        print_obj(item, lib, fmt=fmt)
+                    for item in self._missing(album):
+                        print_(format(item, fmt))
 
         self._command.func = _miss
         return [self._command]
+
+    def _missing(self, album):
+        """Query MusicBrainz to determine items missing from `album`.
+        """
+        item_mbids = map(lambda x: x.mb_trackid, album.items())
+        if len([i for i in album.items()]) < album.albumtotal:
+            # fetch missing items
+            # TODO: Implement caching that without breaking other stuff
+            album_info = hooks.album_for_mbid(album.mb_albumid)
+            for track_info in getattr(album_info, 'tracks', []):
+                if track_info.track_id not in item_mbids:
+                    item = _item(track_info, album_info, album.id)
+                    self._log.debug(u'track {0} in album {1}',
+                                    track_info.track_id, album_info.album_id)
+                    yield item
