@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # This file is part of beets.
 # Copyright 2015, Fabrice Laporte, Yevgeny Bezman, and Adrian Sampson.
 #
@@ -27,7 +28,6 @@ from beets import logging
 from beets import ui
 from beets.plugins import BeetsPlugin
 from beets.util import syspath, command_output, displayable_path
-from beets import config
 
 
 # Utilities.
@@ -317,6 +317,7 @@ class CommandBackend(Backend):
         the album gain
         """
         if len(items) == 0:
+            self._log.debug('no supported tracks to analyze')
             return []
 
         """Compute ReplayGain values and return a list of results
@@ -388,6 +389,12 @@ class GStreamerBackend(Backend):
         self._res = self.Gst.ElementFactory.make("audioresample", "res")
         self._rg = self.Gst.ElementFactory.make("rganalysis", "rg")
 
+        if self._src is None or self._decbin is None or self._conv is None \
+           or self._res is None or self._rg is None:
+            raise FatalReplayGainError(
+                "Failed to load required GStreamer plugins"
+            )
+
         # We check which files need gain ourselves, so all files given
         # to rganalsys should have their gain computed, even if it
         # already exists.
@@ -515,10 +522,9 @@ class GStreamerBackend(Backend):
         err, debug = message.parse_error()
         f = self._src.get_property("location")
         # A GStreamer error, either an unsupported format or a bug.
-        self._error = \
-            ReplayGainError(u"Error {0} - {1} on file {2}".format(err,
-                                                                  debug,
-                                                                  f))
+        self._error = ReplayGainError(
+            "Error {0!r} - {1!r} on file {2!r}".format(err, debug, f)
+        )
 
     def _on_tag(self, bus, message):
         tags = message.parse_tag()
@@ -694,6 +700,23 @@ class AudioToolsBackend(Backend):
         """
         return [self._compute_track_gain(item) for item in items]
 
+    def _title_gain(self, rg, audiofile):
+        """Get the gain result pair from PyAudioTools using the `ReplayGain`
+        instance `rg` for the given `audiofile`.
+
+        Wraps `rg.title_gain(audiofile.to_pcm())` and throws a
+        `ReplayGainError` when the library fails.
+        """
+        try:
+            # The method needs an audiotools.PCMReader instance that can
+            # be obtained from an audiofile instance.
+            return rg.title_gain(audiofile.to_pcm())
+        except ValueError as exc:
+            # `audiotools.replaygain` can raise a `ValueError` if the sample
+            # rate is incorrect.
+            self._log.debug('error in rg.title_gain() call: {}', exc)
+            raise ReplayGainError('audiotools audio data error')
+
     def _compute_track_gain(self, item):
         """Compute ReplayGain value for the requested item.
 
@@ -701,11 +724,10 @@ class AudioToolsBackend(Backend):
         """
         audiofile = self.open_audio_file(item)
         rg = self.init_replaygain(audiofile, item)
-        # Each call to title_gain on a replaygain object return peak and gain
+
+        # Each call to title_gain on a ReplayGain object returns peak and gain
         # of the track.
-        # Note that the method needs an audiotools.PCMReader instance that can
-        # be obtained from an audiofile instance.
-        rg_track_gain, rg_track_peak = rg.title_gain(audiofile.to_pcm())
+        rg_track_gain, rg_track_peak = rg._title_gain(rg, audiofile)
 
         self._log.debug(u'ReplayGain for track {0} - {1}: {2:.2f}, {3:.2f}',
                         item.artist, item.title, rg_track_gain, rg_track_peak)
@@ -728,7 +750,7 @@ class AudioToolsBackend(Backend):
         track_gains = []
         for item in album.items():
             audiofile = self.open_audio_file(item)
-            rg_track_gain, rg_track_peak = rg.title_gain(audiofile.to_pcm())
+            rg_track_gain, rg_track_peak = self._title_gain(rg, audiofile)
             track_gains.append(
                 Gain(gain=rg_track_gain, peak=rg_track_peak)
             )
@@ -903,7 +925,7 @@ class ReplayGainPlugin(BeetsPlugin):
         def func(lib, opts, args):
             self._log.setLevel(logging.INFO)
 
-            write = config['import']['write'].get(bool)
+            write = ui.should_write()
 
             if opts.album:
                 for album in lib.albums(ui.decargs(args)):
